/*
 *	A quick hack to save or restore the CMOS RAM values from 0x0 to 0x7f.
 *	Note that some CMOS are only 0x40 in size, so edit accordingly. 
 *	
 *	Upon restore, values associated from the clock are left untouched.
 *	
 *
 *					Paul Gortmaker	 02/2001
 */

#define CMOS_SIZE 0x80

#include <stdio.h>
#include <string.h>
#include <asm/io.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

/*
 * <linux/rtc.h> was <linux/mc146818rtc.h> on kernels prior to 2.2.19, so
 * just define CMOS_READ/WRITE here independently and avoid the hassle.
 */

#define RTC_PORT(x)     (0x70 + (x))
#define CMOS_READ(addr) ({ \
outb_p((addr),RTC_PORT(0)); \
inb_p(RTC_PORT(1)); \
})
#define CMOS_WRITE(val, addr) ({ \
outb_p((addr),RTC_PORT(0)); \
outb_p((val),RTC_PORT(1)); \
})

#define BCD_TO_BIN(val) ((val)=((val)&15) + ((val)>>4)*10)
 
void usage(char *name){
	fprintf(stderr, "Usage: %s [-w] bkupfile\n"
		"\tstores CMOS contents into bkupfile\n"
		"\t-w: write bkupfile contents into CMOS\n", name);
	exit(-EINVAL);
}

int main(int argc, char** argv) {

int i;
unsigned char val, write=0, cmos_buf[CMOS_SIZE];
FILE* f;

if (argc == 3) {
	if (strncmp(argv[1], "-w", 2)) usage(argv[0]);
	write = 1;
	fprintf(stderr, "Restoring CMOS from file \"%s\"\n", argv[2]);
}
else if (argc == 2) {
	if (strncmp(argv[1], "-w", 2) == 0) usage(argv[0]);
	fprintf(stderr, "Backing up CMOS contents to file \"%s\"\n", argv[1]);
}
else usage(argv[0]);

if (write) {
	struct stat sbuf;

	i=stat(argv[2], &sbuf);
	if (i == -1) {
		perror("stat");
		exit(-errno);
	}
	if (sbuf.st_size != CMOS_SIZE) {
		fprintf(stderr, "File %s is wrong size! (%ld, should be %d)\n", argv[2], sbuf.st_size, CMOS_SIZE);
		exit(-EINVAL);
	}

	f=fopen(argv[2], "r");
	if (f == NULL) {
		perror("fopen");
		exit(-errno);
	}

	i = fread(cmos_buf, 1, CMOS_SIZE, f);
	if (i != CMOS_SIZE) {
		fprintf(stderr, "read of %s failed\n", argv[2]);
		exit(-EBADF);
	}

	fprintf(stderr, "Date stored in CMOS backup: %.2d:%.2d:%.2d, %.2d/%.2d/%.2d (d/m/y)\n",
		BCD_TO_BIN(cmos_buf[4]),
		BCD_TO_BIN(cmos_buf[2]),
		BCD_TO_BIN(cmos_buf[0]),
		BCD_TO_BIN(cmos_buf[7]),
		BCD_TO_BIN(cmos_buf[8]),
		BCD_TO_BIN(cmos_buf[9])%100);
}
else {
	f=fopen(argv[1], "w");
	if (f == NULL) {
		perror("fopen");
		exit(-errno);
	}
}

val= iopl(3);
if (val) {
	perror("iopl");
	exit(errno);
}

for (i = 0; i < CMOS_SIZE; i++) {
	if (write) {
		for (i=0; i<CMOS_SIZE; i++) 
			/* skip RTC clock vals */
			if (i > 0x0d) CMOS_WRITE(cmos_buf[i], i);
	}
	else {
		int j;
		val = CMOS_READ(i);
		j = fwrite(&val, 1, 1, f);
		if (j != 1) {
			fprintf(stderr, "write failed\n");
			exit(-ENOSPC);
		}
	}
}

iopl(0);

if (fclose(f)) {
	perror("fclose");
	exit(-errno);
}
	
exit(0);

} 
